
module.exports = new Class
({
	Extends: Vn.Object

	,tpvOrder: null
	,tpvStatus: null

	,check: function(callback) {
		this.tpvOrder = Vn.Hash.get('tpvOrder');
		this.tpvStatus = Vn.Hash.get('tpvStatus');

		if (this.tpvStatus) {
			var batch = new Sql.Batch();
			batch.addValue('transaction', this.tpvOrder);
			batch.addValue('status', this.tpvStatus);
		
			var query = 'CALL myTpvTransaction_end(#transaction, #status)';
			this.conn.execQuery(query, null, batch);
		}
		
		if (callback)
			callback(this, this.tpvOrder, this.tpvStatus);
	}

	,pay: function(amount, company) {
		this._realPay(amount * 100, company);
	}

	,_realPay: function(amount, company) {
		if (isNumeric(amount) && amount > 0) {
			var params = {
				 amount: parseInt(amount)
				,urlOk: this._makeUrl('ok')
				,urlKo: this._makeUrl('ko')
				,company: company
			};
		
			this.conn.send('tpv/transaction', params,
				this._onTransactionStart.bind(this));
		} else
			Htk.Toast.showError(_('AmountError'));
	}

	,_onTransactionStart: function(json) {
		if (json) {
			var postValues = json.postValues;

			var form = document.createElement('form');
			form.method = 'POST';
			form.action = json.url;
			document.body.appendChild(form);
			
			for (var field in postValues) {
				var input = document.createElement('input');
				input.type = 'hidden';
				input.name = field;
				form.appendChild(input);
				
				if (postValues[field])
					input.value = postValues[field];
			}
			
			form.submit();
		} else
			Htk.Toast.showWarning(_('PayError'));
	}
	
	,retryPay: function() {
		var batch = new Sql.Batch();
		batch.addValue('transaction', parseInt(this.tpvOrder));

		var query = 'SELECT t.amount, m.companyFk '
			+'FROM myTpvTransaction t '
			+'JOIN tpvMerchant m ON m.id = t.merchantFk '
				+'WHERE t.id = #transaction';
		this.conn.execQuery(query,
			this._onRetryPayDone.bind(this), batch);
	}
	
	,_onRetryPayDone: function(resultSet) {
		var res = resultSet.fetchResult();
		
		if (res.next())
			this._realPay(res.get('amount'), res.get('companyFk'));
		else
			Htk.Toast.showError(_('AmountError'));
	}

	,_makeUrl: function(status) {
		var path = location.protocol +'//'+ location.host;
		path += location.port ? ':'+ location.port : '';
		path += location.pathname;
		path += location.search ? location.search : '';
		path += Vn.Hash.make({
			form: 'ecomerce/orders',
			tpvStatus: status,
			tpvOrder: '_transactionId_'
		}, true);

		return path;
	}
});

function isNumeric(n) {
	return !isNaN(parseFloat(n)) && isFinite(n);
}
